<!DOCTYPE html>
<html lang="en-US">
  <head>
    <title>Documentation for <%= version%></title>
    <link href="<%= version%>/style.css" media="screen" rel="stylesheet" type="text/css">
  </head>
  <body>
    <div id="container">
      <div id="operations">
        <h3>API Operations</h3>
      </div>
      <div id="content">
        <h1>Documentation for <%= version%></h1>
      </div>
    </div>
  </body>
</html>
