require 'test_helper'

class <%= module_name %>::<%= class_name%>ControllerTest < ActionDispatch::IntegrationTest

  # Replace this with your real tests.
  test "the truth" do
    assert true
  end
end
